import Position from '../position/Position'

const { Cesium } = BC.Namespace

class Fire {
  constructor(pos, opt) {
    if (!(pos instanceof Position)) {
      pos = new Position()
    }
    this.position = pos
    this.opt = opt
    this.entity = null
    this.pointEntity = null
  }

  init() {
    if (!window.viewer) {
      console.error('地球未初始化，火加载失败')
    }
    //粒子系统的起点，发射源
    var staticPosition = Cesium.Cartesian3.fromDegrees(
      this.position.lng,
      this.position.lat,
      this.position.alt
    )
    this.pointEntity = viewer.entities.add({
      position: staticPosition
    })
    // viewer.flyTo(entity44)

    this.entity = viewer.scene.primitives.add(
      new Cesium.ParticleSystem({
        image: this.opt?.image || '/libs/resources/static/fire.png',
        startColor:
          this.opt && this.opt.startColor
            ? Cesium.Color.fromBytes(
                this.opt.startColor[0],
                this.opt.startColor[0],
                this.opt.startColor[0],
                this.opt.startColor[0]
              )
            : Cesium.Color.RED.withAlpha(0.7),
        endColor:
          this.opt && this.opt.endColor
            ? Cesium.Color.fromBytes(
                this.opt.endColor[0],
                this.opt.endColor[0],
                this.opt.endColor[0],
                this.opt.endColor[0]
              )
            : Cesium.Color.YELLOW.withAlpha(0.3),
        startScale: this.opt?.startScale || 0,
        endScale: this.opt?.endScale || 5,
        //设定粒子寿命可能持续时间的最小限值(以秒为单位)，在此限值之上将随机选择粒子的实际寿命。
        minimumParticleLife: this.opt?.minimumParticleLife || 1,
        maximumParticleLife: this.opt?.maximumParticleLife || 6,
        minimumSpeed: this.opt?.minimumSpeed || 1,
        maximumSpeed: this.opt?.maximumSpeed || 4,
        imageSize: new Cesium.Cartesian2(
          this.opt?.imageSize?.[0] || 30,
          this.opt?.imageSize?.[1] || 30
        ),
        sizeInMeters: this.opt?.sizeInMeters || true,
        emissionRate: this.opt?.emissionRate || 4, //火焰大小
        lifetime: this.opt?.lifetime || 160.0,
        emitter: new Cesium.CircleEmitter(5.0),
        modelMatrix: this.firecomputeModelMatrix(this.pointEntity),
        emitterModelMatrix: this.firecomputeEmitterModelMatrix()
      })
    )
  }

  firecomputeModelMatrix(entity) {
    var position = Cesium.Property.getValueOrUndefined(entity.position)
    let modelMatrix = Cesium.Transforms.eastNorthUpToFixedFrame(position)
    return modelMatrix
  }

  firecomputeEmitterModelMatrix() {
    let hpr = Cesium.HeadingPitchRoll.fromDegrees(
      this.position.heading,
      this.position.pitch,
      this.position.roll
    )
    let trs = new Cesium.TranslationRotationScale()
    trs.translation = Cesium.Cartesian3.fromElements(2.5, 4, 1)
    trs.rotation = Cesium.Quaternion.fromHeadingPitchRoll(hpr)
    let result = Cesium.Matrix4.fromTranslationRotationScale(trs)
    return result
  }

  start() {
    if (this.entity) {
      this.entity.show = true
      return
    }
    this.init()
  }

  stop() {
    this.entity ? (this.entity.show = false) : ''
  }

  uninstall() {
    if (!this.entity) return
    viewer.scene.primitives.remove(this.entity)
    this.entity = null
  }
  zoomTo() {
    viewer.zoomTo(this.entity)
  }
}

export default Fire
